@@ -20,39 +20,30 @@
 
 /**
  * SECTION:element-vorbisparse
- * @short_description: parses vorbis streams 
  * @see_also: vorbisdec, oggdemux, theoraparse
  *
- * <refsect2>
- * <para>
  * The vorbisparse element will parse the header packets of the Vorbis
  * stream and put them as the streamheader in the caps. This is used in the
  * multifdsink case where you want to stream live vorbis streams to multiple
  * clients, each client has to receive the streamheaders first before they can
  * consume the vorbis packets.
- * </para>
- * <para>
+ *
  * This element also makes sure that the buffers that it pushes out are properly
  * timestamped and that their offset and offset_end are set. The buffers that
  * vorbisparse outputs have all of the metadata that oggmux expects to receive,
  * which allows you to (for example) remux an ogg/vorbis file.
- * </para>
+ *
+ * <refsect2>
  * <title>Example pipelines</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch -v filesrc location=sine.ogg ! oggdemux ! vorbisparse ! fakesink
- * </programlisting>
- * This pipeline shows that the streamheader is set in the caps, and that each
+ * ]| This pipeline shows that the streamheader is set in the caps, and that each
  * buffer has the timestamp, duration, offset, and offset_end set.
- * </para>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch filesrc location=sine.ogg ! oggdemux ! vorbisparse \
  *            ! oggmux ! filesink location=sine-remuxed.ogg
- * </programlisting>
- * This pipeline shows remuxing. sine-remuxed.ogg might not be exactly the same
+ * ]| This pipeline shows remuxing. sine-remuxed.ogg might not be exactly the same
  * as sine.ogg, but they should produce exactly the same decoded data.
- * </para>
  * </refsect2>
  *
  * Last reviewed on 2006-04-01 (0.10.4.1)
@@ -67,25 +58,13 @@
 GST_DEBUG_CATEGORY_EXTERN (vorbisparse_debug);
 #define GST_CAT_DEFAULT vorbisparse_debug
 
-static GstElementDetails vorbis_parse_details = {
+static const GstElementDetails vorbis_parse_details = {
   "VorbisParse",
   "Codec/Parser/Audio",
   "parse raw vorbis streams",
   "Thomas Vander Stichele <thomas at apestaart dot org>"
 };
 
-/* Filter signals and args */
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
-
-enum
-{
-  ARG_0
-};
-
 static GstStaticPadTemplate vorbis_parse_sink_factory =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -111,6 +90,8 @@ static gboolean vorbis_parse_src_query (GstPad * pad, GstQuery * query);
 static gboolean vorbis_parse_convert (GstPad * pad,
     GstFormat src_format, gint64 src_value,
     GstFormat * dest_format, gint64 * dest_value);
+static GstFlowReturn vorbis_parse_parse_packet (GstVorbisParse * parse,
+    GstBuffer * buf);
 
 static void
 gst_vorbis_parse_base_init (gpointer g_class)
@@ -130,6 +111,8 @@ gst_vorbis_parse_class_init (GstVorbisParseClass * klass)
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
 
   gstelement_class->change_state = vorbis_parse_change_state;
+
+  klass->parse_packet = GST_DEBUG_FUNCPTR (vorbis_parse_parse_packet);
 }
 
 static void
@@ -137,13 +120,16 @@ gst_vorbis_parse_init (GstVorbisParse * parse, GstVorbisParseClass * g_class)
 {
   parse->sinkpad =
       gst_pad_new_from_static_template (&vorbis_parse_sink_factory, "sink");
-  gst_pad_set_chain_function (parse->sinkpad, vorbis_parse_chain);
-  gst_pad_set_event_function (parse->sinkpad, vorbis_parse_sink_event);
+  gst_pad_set_chain_function (parse->sinkpad,
+      GST_DEBUG_FUNCPTR (vorbis_parse_chain));
+  gst_pad_set_event_function (parse->sinkpad,
+      GST_DEBUG_FUNCPTR (vorbis_parse_sink_event));
   gst_element_add_pad (GST_ELEMENT (parse), parse->sinkpad);
 
   parse->srcpad =
       gst_pad_new_from_static_template (&vorbis_parse_src_factory, "src");
-  gst_pad_set_query_function (parse->srcpad, vorbis_parse_src_query);
+  gst_pad_set_query_function (parse->srcpad,
+      GST_DEBUG_FUNCPTR (vorbis_parse_src_query));
   gst_element_add_pad (GST_ELEMENT (parse), parse->srcpad);
 }
 
@@ -223,6 +209,7 @@ vorbis_parse_push_headers (GstVorbisParse * parse)
   packet.granulepos = GST_BUFFER_OFFSET_END (outbuf);
   packet.packetno = 1;
   packet.e_o_s = 0;
+  packet.b_o_s = 1;
   vorbis_synthesis_headerin (&parse->vi, &parse->vc, &packet);
   parse->sample_rate = parse->vi.rate;
   outbuf1 = outbuf;
@@ -233,6 +220,7 @@ vorbis_parse_push_headers (GstVorbisParse * parse)
   packet.granulepos = GST_BUFFER_OFFSET_END (outbuf);
   packet.packetno = 2;
   packet.e_o_s = 0;
+  packet.b_o_s = 0;
   vorbis_synthesis_headerin (&parse->vi, &parse->vc, &packet);
   outbuf2 = outbuf;
 
@@ -242,6 +230,7 @@ vorbis_parse_push_headers (GstVorbisParse * parse)
   packet.granulepos = GST_BUFFER_OFFSET_END (outbuf);
   packet.packetno = 3;
   packet.e_o_s = 0;
+  packet.b_o_s = 0;
   vorbis_synthesis_headerin (&parse->vi, &parse->vc, &packet);
   outbuf3 = outbuf;
 
@@ -401,15 +390,10 @@ vorbis_parse_queue_buffer (GstVorbisParse * parse, GstBuffer * buf)
 }
 
 static GstFlowReturn
-vorbis_parse_chain (GstPad * pad, GstBuffer * buffer)
+vorbis_parse_parse_packet (GstVorbisParse * parse, GstBuffer * buf)
 {
   GstFlowReturn ret;
-  GstBuffer *buf;
-  GstVorbisParse *parse;
 
-  parse = GST_VORBIS_PARSE (gst_pad_get_parent (pad));
-
-  buf = GST_BUFFER (buffer);
   parse->packetno++;
 
   if (parse->packetno <= 3) {
@@ -424,11 +408,23 @@ vorbis_parse_chain (GstPad * pad, GstBuffer * buffer)
     ret = vorbis_parse_queue_buffer (parse, buf);
   }
 
-  gst_object_unref (parse);
-
   return ret;
 }
 
+static GstFlowReturn
+vorbis_parse_chain (GstPad * pad, GstBuffer * buffer)
+{
+  GstVorbisParseClass *klass;
+  GstVorbisParse *parse;
+
+  parse = GST_VORBIS_PARSE (GST_PAD_PARENT (pad));
+  klass = GST_VORBIS_PARSE_CLASS (G_OBJECT_GET_CLASS (parse));
+
+  g_assert (klass->parse_packet != NULL);
+
+  return klass->parse_packet (parse, buffer);
+}
+
 static gboolean
 vorbis_parse_queue_event (GstVorbisParse * parse, GstEvent * event)
 {
@@ -575,8 +571,8 @@ vorbis_parse_src_query (GstPad * pad, GstQuery * query)
 
       gst_query_set_position (query, format, value);
 
-      GST_LOG_OBJECT (parse,
-          "query %u: peer returned granulepos: %llu - we return %llu (format %u)",
+      GST_LOG_OBJECT (parse, "query %p: peer returned granulepos: %"
+          G_GUINT64_FORMAT " - we return %" G_GUINT64_FORMAT " (format %u)",
           query, granulepos, value, format);
 
       break;
